@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.105';
+our $VERSION = '0.106';
 use 5.006;
 use strict;
 use warnings;
@@ -16,7 +16,7 @@ BEGIN {
 	$JSON = JSON->new->utf8->canonical(1)->allow_blessed->convert_blessed;
 	%PRIVATE = (
 		map { $_ => 1 } 
-		qw/new run logger logInfo logError raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
+		qw/new run logger logInfo logError delayed_response content_type raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
 	);
 }
 
@@ -43,6 +43,7 @@ sub run {
 		logout => 'logout',
 		auth => 'auth',
 		insecure_session => 0,
+		content_type => 'application/json',
 		%args
 	);
 
@@ -62,6 +63,7 @@ sub run {
 
 	$j->{_application_has_dispatcher} = !! $j->{_application}->can('dispatch');
 	$j->{_application_has_response_handler} = !! $j->{_application}->can('response_handle');
+	$j->{_application_has_delayed_response_handler} = !! $j->{_application}->can('delayed_response_handle');
 	
 	my $content_type = $j->request->content_type;
 	if ($content_type && $content_type =~ m/application\/json/) {
@@ -87,6 +89,12 @@ sub run {
 		$j->sid = $h->hexdigest;
 	}
 
+	$j->cookie = {
+		value => $j->is_logout ? "" : $j->sid,
+		path  => $j->request->uri,
+		secure => !$j->{_insecure_session},
+	};
+
 	my $auth = $j->{_auth};
 	my ($session) = $j->_dispatch($auth, $pkg->new());
 	my $req = $j->params->req;
@@ -100,6 +108,9 @@ sub run {
 	$j->response->authenticated = \1;
 	$j->session = $session;
 
+	$j->cookie->expires = (ref $j->session && $j->session->expires) || (time + 24 * 60 * 60) 
+		if (!$j->cookie->expires);
+
 	($j->is_login, $j->is_logout) = (
 		$j->{_login} eq $req,
 		$j->{_logout} eq $req
@@ -118,22 +129,53 @@ sub run {
 		return $j->_response($j->response);
 	}
 
-	my $cookie = {
-		value => $j->is_logout ? "" : $j->sid,
-		path  => $j->request->uri,
-		secure => !$j->{_insecure_session},
-		expires => $j->is_logout ? time : ((ref $j->session && $j->session->expires) || time + 24 * 60 * 60),
-	};
+	return $j->_response($j->response, $j->content_type, $j->cookie);
+}
 
-	return $j->_response($j->response, $cookie);
+sub delayed_response {
+	my ($self, $response, $ct, $cookie, $status) = @_;
+	$cookie ||= $self->cookie;
+	$status ||= 200;
+	$ct ||= 'application/json';
+	return $self->{_application}->delayed_response_handle(
+		$self, $response, $ct, $cookie, $status
+	) if $self->{_application_has_delayed_response_handler};
+	$self->{_delayed_response} = sub {
+		my $responder = shift;
+		my $res = $self->request->new_response($self->response->{status_code} ||= $status);
+		$res->cookies->{sid} = $cookie if $cookie;
+		$res->content_type($ct);
+		$res = [splice @{$res->finalize}, 0, 2];
+		my $writer = $responder->($res);
+		$response = eval { $response->($writer); };
+		if ($@ || $self->response->error) {
+			$res->[0] = $self->response->status_code || 500;
+			$self->raiseError($@) if $@;
+			push @{$res}, [$self->response->serialize];
+			return $responder->($res);
+		}
+		elsif ($response) {
+			$writer->write($response->serialize);
+		}
+		$writer->close;
+	};
+	return $self;
+}
+
+sub content_type { 
+	$_[0]->{_content_type} = $_[1] if $_[1];
+	return $_[0]->{_content_type};
 }
 
 sub _response {
-	my ($self, $response_body, $cookie, $status) = @_;
+	my ($self, $response_body, $ct, $cookie, $status) = @_;
 	return $self->{_application}->response_handle(@_) if $self->{_application_has_response_handler};
-	my $res = $self->request->new_response($response_body->status_code ||= $status ||= 200);
+	$ct ||= 'application/json';
+	my $res = $self->{_delayed_response};
+	return $res if ($res); 
+	$res = $self->request->new_response($response_body->status_code ||= $status ||= 200);
 	$res->cookies->{sid} = $cookie if $cookie;
-	$res->content_type('application/json');
+	$res->content_type($ct);
 	$res->body($response_body->serialize());
 	return $res->finalize;
 }
@@ -276,7 +318,7 @@ Terse - Lightweight JSON APIs.
 
 =head1 VERSION
 
-Version 0.105
+Version 0.106
 
 =cut
 
@@ -303,6 +345,21 @@ Version 0.105
 		$t->response->hello = "world";
 	}
 
+	sub delayed_hello_world {
+		my ($self, $t) = @_;
+		$t->delayed_response(sub {
+			if ($t->params->throw_error) {
+				$t->logError('throw 500 error which is also logged', 500);
+				return;
+			}
+
+			... do something which takes a long time ...
+
+			$t->response->hello = "world";
+			return $t->response;
+		});
+	}
+
 	.... MyAPI.psgi ...
 
 	use Terse;
@@ -329,7 +386,7 @@ Version 0.105
 
 	plackup MyAPI.psgi
 
-	GET http://localhost:5000/?req=hello_world
+	GET http://localhost:5000/?req=delayed_hello_world
 	# {"authenticated":1,"error":false,"errors":[],"hello":"world","status_code":200}
 	GET http://localhost:5000/?req=hello_world&not=1 
 	# {"authenticated":0,"error":true,"errors":["Invalid request"],"status_code":400}
@@ -395,6 +452,7 @@ Returns the Plack::Request.
 
 =cut
 
+
 =head2 session
 
 Retrieve current session data, set in your auth or login methods
@@ -469,6 +527,17 @@ Encode a perl struct as a JSON string.
 
 =cut
 
+=head2 delayed_response
+
+Delay the response for non-blocking I/O based server streaming or long-poll Comet push technology.
+
+	$terse->delayed_response(sub {
+		$terse->response->test = 'okay';
+		return $terse->response;
+	});
+
+=cut
+
 =head1 AUTHOR
 
 LNATION, C<< <email at lnation.org> >>
