@@ -121,13 +121,43 @@ sub PARSE_ATTRIBUTES {
 	return %attr;
 }
 
+sub delayed_response_handle {
+	my ($self, $t,  $response, $ct, $cookie, $status) = @_;
+	$t->{_delayed_response} = sub {
+		my $responder = shift;
+		my $res = $t->request->new_response($t->response->status_code ||= $status);
+		$res->cookies->{sid} = $cookie if $cookie;
+		my $view = $t->view($self->response_view);
+		$res->content_type( $view ? $view->content_type($ct) : $ct);
+		$res = [splice @{$res->finalize}, 0, 2];
+		my $writer = $responder->($res);
+		$response = eval { $response->($writer); };
+		if ($@ || $t->response->error) {
+			$res->[0] = $t->response->status_code || 500;
+			$t->raiseError($@) if #@;
+			push @{$res}, [$t->response->serialize];
+			return $responder->($res);
+		}
+		elsif ($response) {
+			$view = $t->view($self->response_view);
+			$writer->write($view ? [$view->render($t, $response)]->[1] : $response->serialize);
+		}
+		$writer->close;
+	};
+	return $t;
+}
+
+
 sub response_handle {
-	my ($self, $t, $response_body, $cookie, $status) = @_;
-	my $res = $t->request->new_response($response_body->status_code ||= $status ||= 200);
+	my ($self, $t, $response_body, $ct, $cookie, $status) = @_;
+	$ct ||= 'application/json';
+	my $res = $t->{_delayed_response};
+	return $res if ($res); 
+	$res = $t->request->new_response($response_body->status_code ||= $status ||= 200);
 	$res->cookies->{sid} = $cookie if $cookie;
 	my ($content_type, $body) = $self->views->{$self->response_view} 
 		? $t->view($self->response_view)->render($t, $response_body) 
-		: ('application/json', $response_body->serialize());
+		: ($ct, $response_body->serialize());
 	$res->content_type($content_type);
 	$res->body($body);
 	return $res->finalize;
@@ -165,6 +195,9 @@ sub dispatch {
 		$t->logError('No callback found to dispatch the request', 400);
 		return;
 	}
+	if ($dispatch->{delayed}) {
+		return $t->delayed_response(sub { $self->$callback($t, @params); $t->response; });
+	}
 	return $self->$callback($t, @params);
 }
 
@@ -199,7 +232,7 @@ Terse::Controller - controllers made simple.
 
 =head1 VERSION
 
-Version 0.105
+Version 0.106
 
 =cut
 
@@ -218,10 +251,10 @@ Version 0.105
 	}
 
 	sub auth :get :post {
-		return 1;
+		return $_[1]->delayed_response(sub { ... });;
 	}
 
-	sub purchase :get {
+	sub purchase :get :delayed { # delayed attribute is the same as the above 
 		... #1
 	}
 	
