@@ -8,8 +8,10 @@ sub build_controller {
 	return $self;
 }
 
-sub hidden {
-
+sub hidden :get :path(app/hidden) :delayed {
+	my ($self, $t) = @_;
+	$t->response->hits = '1';
+	$t->response->test = "kaput";
 }
 
 sub hello :any(hello_world) {
